import { Store } from 'vuex';

var DecoratorType;
(function (DecoratorType) {
    DecoratorType[DecoratorType["STATE"] = 1] = "STATE";
    DecoratorType[DecoratorType["ACTION"] = 2] = "ACTION";
    DecoratorType[DecoratorType["MUTATION"] = 3] = "MUTATION";
    DecoratorType[DecoratorType["GETTER"] = 4] = "GETTER";
    DecoratorType[DecoratorType["MODULE"] = 5] = "MODULE";
})(DecoratorType || (DecoratorType = {}));

var KEY = '__decorators__';
function setDecorator(target, propertyName, type, options) {
    var decorators = getDecorators(target);
    if (decorators) {
        decorators.set(propertyName, {
            options: options,
            propertyName: propertyName,
            type: type
        });
    }
}
function getDecorators(target) {
    var ctor = typeof target === 'function' ? target : target.constructor;
    var proto = ctor.prototype;
    var decorators = proto[KEY];
    if (!proto.hasOwnProperty(KEY) && decorators) {
        proto[KEY] = new Map(decorators.entries());
        return proto[KEY];
    }
    else if (!decorators) {
        proto[KEY] = new Map();
        return proto[KEY];
    }
    return decorators;
}

function State() {
    return function (target, propertyName) {
        setDecorator(target, propertyName, DecoratorType.STATE);
    };
}
function Module() {
    return function (target, propertyName) {
        setDecorator(target, propertyName, DecoratorType.MODULE);
    };
}
function Getter() {
    return function (target, propertyName, descriptor) {
        setDecorator(target, propertyName, DecoratorType.GETTER);
    };
}
function Mutation() {
    return function (target, propertyName, descriptor) {
        setDecorator(target, propertyName, DecoratorType.MUTATION);
    };
}
function Action() {
    return function (target, propertyName, descriptor) {
        setDecorator(target, propertyName, DecoratorType.ACTION);
    };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function createVuexStore(pModule, pOptions) {
    var storeProvider = {
        store: undefined
    };
    var userOptions = __assign({ modules: {}, plugins: [], strict: false }, pOptions);
    var options = transformRecursive(storeProvider, pModule);
    options.strict = userOptions.strict;
    options.modules = __assign({}, options.modules, userOptions.modules);
    options.plugins = userOptions.plugins;
    var store = new Store(options);
    store.__root__ = pModule;
    storeProvider.store = store;
    return store;
}
function useStore(vuexStore) {
    var typedStore = vuexStore;
    return typedStore.__root__;
}
function transformRecursive(pStoreProvider, pModule, pNamespace) {
    if (pNamespace === void 0) { pNamespace = []; }
    var storeOptions = {
        actions: {},
        getters: {},
        modules: {},
        mutations: {},
        namespaced: true,
        state: {}
    };
    var builder = {
        namespaces: pNamespace,
        options: storeOptions,
        provider: pStoreProvider
    };
    var constructor = pModule.constructor;
    var decorators = getDecorators(constructor);
    decorators.forEach(function (decorator) {
        var type = decorator.type, propertyName = decorator.propertyName;
        switch (type) {
            case DecoratorType.STATE:
                return bindState(builder, pModule, propertyName);
            case DecoratorType.GETTER:
                return bindGetter(builder, pModule, propertyName);
            case DecoratorType.MUTATION:
                return bindMutation(builder, pModule, propertyName);
            case DecoratorType.ACTION:
                return bindAction(builder, pModule, propertyName);
            case DecoratorType.MODULE:
                return bindModule(builder, pModule, propertyName);
                break;
        }
    });
    return storeOptions;
}
function getState(pState, pNamespace) {
    var state = pState;
    for (var _i = 0, pNamespace_1 = pNamespace; _i < pNamespace_1.length; _i++) {
        var namespace = pNamespace_1[_i];
        state = state[namespace];
    }
    return state;
}
function bindState(pBuilder, pModule, propertyName) {
    var value = pModule[propertyName];
    var provider = pBuilder.provider;
    var options = pBuilder.options;
    Object.defineProperty(pModule, propertyName, {
        get: function () {
            var state = provider.store
                ? getState(provider.store.state, pBuilder.namespaces)
                : options.state;
            return state[propertyName];
        },
        set: function (val) {
            var state = provider.store
                ? getState(provider.store.state, pBuilder.namespaces)
                : options.state;
            state[propertyName] = val;
        }
    });
    pModule[propertyName] = value;
}
function getGetter(pModule, propertyName) {
    var getter = pModule.__lookupGetter__(propertyName);
    if (!getter) {
        var className = pModule.constructor.name;
        throw {
            code: 'getter_not_found',
            message: "Could not find getter \"" + propertyName + "\" for module " + className,
            params: {
                className: className,
                propertyName: propertyName
            }
        };
    }
    var boundGetter = getter.bind(pModule);
    return function (state) { return boundGetter(); };
}
function bindGetter(pBuilder, pModule, propertyName) {
    var options = pBuilder.options;
    var provider = pBuilder.provider;
    options.getters[propertyName] = getGetter(pModule, propertyName);
    var getterName = pBuilder.namespaces.join('/') + '/' + propertyName;
    Object.defineProperty(pModule, propertyName, {
        get: function () {
            return provider.store.getters[getterName];
        }
    });
}
function getMutation(pModule, propertyName) {
    var mutation = pModule[propertyName];
    var boundMutation = mutation.bind(pModule);
    return function (state, payload) { return boundMutation(payload); };
}
function bindMutation(pBuilder, pModule, propertyName) {
    var options = pBuilder.options;
    var provider = pBuilder.provider;
    options.mutations[propertyName] = getMutation(pModule, propertyName);
    var mutationName = pBuilder.namespaces.join('/') + '/' + propertyName;
    pModule[propertyName] = function (payload) {
        provider.store.commit(mutationName, payload);
    };
}
function getAction(pModule, propertyName) {
    var action = pModule[propertyName];
    var boundAction = action.bind(pModule);
    return function (state, payload) { return boundAction(payload); };
}
function bindAction(pBuilder, pModule, propertyName) {
    var options = pBuilder.options;
    var provider = pBuilder.provider;
    options.actions[propertyName] = getAction(pModule, propertyName);
    var actionName = pBuilder.namespaces.join('/') + '/' + propertyName;
    pModule[propertyName] = function (payload) {
        provider.store.dispatch(actionName, payload);
    };
}
function bindModule(pBuilder, pModule, propertyName) {
    var subModule = pModule[propertyName];
    if (!subModule) {
        return;
    }
    var moduleOptions = transformRecursive(pBuilder.provider, subModule, pBuilder.namespaces.concat(propertyName));
    var options = pBuilder.options;
    options.modules[propertyName] = moduleOptions;
    Object.defineProperty(pModule, propertyName, {
        get: function () {
            return subModule;
        }
    });
}

export { State, Module, Getter, Mutation, Action, createVuexStore, useStore };
